from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face, Rectangle, RightTriangle
import rocolib.utils.numsym as math


def pyramid(l,w,h):
    def baseangle(x, y):
        x2 = x/2.
        hx = math.sqrt(h*h+x2*x2)
        xa = math.rad2deg(math.arctan2(h,x2))
        ba = math.rad2deg(math.arctan2(hx,y/2.))
        return x2, hx, xa, ba

    l2, hl, la, bla = baseangle(l, w)
    w2, hw, wa, bwa = baseangle(w, l)
    diag = math.sqrt(h*h + w2*w2 + l2+l2)
    da = math.rad2deg(math.arccos(1/math.sqrt((1 + h*h/w2/w2)*(1+h*h/l2/l2))))
    return hl, la, bla, hw, wa, bwa, diag, da


class BoatPoint(FoldedComponent):
    def define(self):
        self.addParameter("width", 50, paramType="length")
        self.addParameter("depth", 25, paramType="length")
        self.addParameter("point", 50, paramType="length", minValue=0)

        self.addEdgeInterface("ledge", "sl.e1", "depth")
        self.addEdgeInterface("cedge", "sc.e2", "width")
        self.addEdgeInterface("redge", "sr.e1", "depth")

        self.addEdgeInterface("edge", ["sl.e1", "sc.e2", "sr.e1"], ["depth", "width", "depth"])

    def assemble(self):
        w = self.getParameter("width")
        d = self.getParameter("depth")
        p = self.getParameter("point")

        hl, la, bla, hw, wa, bwa, diag, da = pyramid(d*2,w,p)

        # Flaps
        ba = (180 - bla - bwa)/2.
        fx = d * math.tan(math.deg2rad(ba))

        fll = Face("", ((fx, 0), (hw, 0), (0, d), (0,0)))
        flc = RightTriangle("", w/2., hl);
        frc = RightTriangle("", hl, w/2.);
        frr = Face("", ((d, 0), (0, hw), (0, fx), (0,0)))

        # Main point
        self.addFace(fll, "lt");
        self.attachEdge("lt.e2", flc, "e1", prefix="lc", angle=da)
        self.attachEdge("lc.e2", frc, "e0", prefix="rc", angle=0)
        self.attachEdge("rc.e1", frr, "e1", prefix="rt", angle=da)

        # Flaps
        fla = RightTriangle("", d, fx);
        flb = RightTriangle("", fx, d);
        self.attachEdge("lt.e3", fla, "e0", prefix="fla", angle=180)
        self.attachEdge("fla.e1", flb, "e1", prefix="flb", angle=-180)

        fra = RightTriangle("", fx, d);
        frb = RightTriangle("", d, fx);
        self.attachEdge("rt.e0", fra, "e2", prefix="fra", angle=180)
        self.attachEdge("fra.e1", frb, "e1", prefix="frb", angle=-180)

        self.addTab("flb.e0", "lt.e0", angle = -174, width=fx/3)
        self.addTab("frb.e2", "rt.e3", angle = -174, width=fx/3)

        # To allow 0 degree attachments at base
        se = Face("", ((0,0), (w/2., 0), (-w/2., 0)))
        self.attachEdge("lc.e0", se, "e0", prefix="sc", angle=90-la)
        self.mergeEdge("rc.e2", "sc.e1", angle=90-la)

        se = Face("", ((0,0), (d,0)))
        self.attachEdge("flb.e2", se, "e0", prefix="sl", angle=90-wa)

        se = Face("", ((0,0), (d,0)))
        self.attachEdge("frb.e0", se, "e0", prefix="sr", angle=90-wa)

if __name__ == "__main__":
    BoatPoint.test()

